Package: customCMPdb
Title: Customize and Query Compound Annotation Database
Description: This package serves as a query interface for important community collections of
    small molecules, while also allowing users to include custom compound collections. 
Version: 1.23.0
Authors@R: 
    c(person("Yuzhu", "Duan", role = c("aut", "cre"), email = "yduan004@ucr.edu"),
      person("Thomas", "Girke", role = "aut", email = "thomas.girke@ucr.edu"))
Depends: R (>= 4.0)
Imports: AnnotationHub, RSQLite, XML, utils, ChemmineR, methods, stats,
        rappdirs, BiocFileCache
Suggests: knitr, rmarkdown, testthat, BiocStyle
License: Artistic-2.0
biocViews: Software, Cheminformatics,AnnotationHubSoftware
Encoding: UTF-8
VignetteBuilder: knitr
URL: https://github.com/yduan004/customCMPdb/
BugReports: https://github.com/yduan004/customCMPdb/issues
RoxygenNote: 7.1.2
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/customCMPdb
git_branch: devel
git_last_commit: 055aeb1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 22:07:26 UTC; biocbuild
Author: Yuzhu Duan [aut, cre],
  Thomas Girke [aut]
Maintainer: Yuzhu Duan <yduan004@ucr.edu>
