# Generated by roxygen2: do not edit by hand

export(addCustomAnnot)
export(buildDrugAgeDB)
export(dbxml2df)
export(defaultAnnot)
export(deleteAnnot)
export(df2SQLite)
export(listAnnot)
export(loadAnnot)
export(loadSDFwithName)
export(processDrugage)
export(queryAnnotDB)
import(AnnotationHub)
import(BiocFileCache)
import(ChemmineR)
import(RSQLite)
importFrom(AnnotationHub,AnnotationHub)
importFrom(XML,xmlParse)
importFrom(XML,xmlRoot)
importFrom(XML,xmlSize)
importFrom(XML,xmlToDataFrame)
importFrom(methods,as)
importFrom(rappdirs,user_cache_dir)
importFrom(stats,na.omit)
importFrom(utils,download.file)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,untar)
importFrom(utils,unzip)
importFrom(utils,write.table)
